@@ -11,24 +11,30 @@
  * the GNU General Public License
  */
  
-#include <asterisk/lock.h>
-#include <asterisk/file.h>
-#include <asterisk/logger.h>
-#include <asterisk/channel.h>
-#include <asterisk/pbx.h>
-#include <asterisk/module.h>
-#include <asterisk/translate.h>
-#include <asterisk/dsp.h>
-#include <asterisk/manager.h>
 #include <string.h>
 #include <stdlib.h>
-#include <stdint.h>
+#include <stdio.h>
+#include <inttypes.h>
 #include <pthread.h>
 #include <errno.h>
 #include <tiffio.h>
 
 #include <spandsp.h>
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision:$")
+
+#include "asterisk/lock.h"
+#include "asterisk/file.h"
+#include "asterisk/logger.h"
+#include "asterisk/channel.h"
+#include "asterisk/pbx.h"
+#include "asterisk/module.h"
+#include "asterisk/translate.h"
+#include "asterisk/dsp.h"
+#include "asterisk/manager.h"
+
 static char *tdesc = "Trivial FAX Receive Application";
 
 static char *app = "RxFAX";
@@ -36,7 +42,7 @@ static char *app = "RxFAX";
 static char *synopsis = "Receive a FAX to a file";
 
 static char *descrip = 
-"  RxFAX(filename[|caller]): Receives a FAX from the channel into the\n"
+"  RxFAX(filename[|caller][|debug]): Receives a FAX from the channel into the\n"
 "given filename. If the file exists it will be overwritten. The file\n"
 "should be in TIFF/F format.\n"
 "The \"caller\" option makes the application behave as a calling machine,\n"
@@ -57,6 +63,20 @@ LOCAL_USER_DECL;
 
 #define MAX_BLOCK_SIZE 240
 
+static void span_message(int level, const char *msg)
+{
+    int ast_level;
+    
+    if (level == SPAN_LOG_WARNING)
+        ast_level = __LOG_WARNING;
+    else if (level == SPAN_LOG_WARNING)
+        ast_level = __LOG_WARNING;
+    else
+        ast_level = __LOG_DEBUG;
+    ast_log(ast_level, __FILE__, __LINE__, __PRETTY_FUNCTION__, msg);
+}
+/*- End of function --------------------------------------------------------*/
+
 static void t30_flush(t30_state_t *s, int which)
 {
     //TODO:
@@ -72,11 +92,11 @@ static void phase_e_handler(t30_state_t *s, void *user_data, int result)
     char buf[11];
     
     chan = (struct ast_channel *) user_data;
-    if (result)
+    if (result == T30_ERR_OK)
     {
-        fax_get_transfer_statistics(s, &t);
-        fax_get_far_ident(s, far_ident);
-        fax_get_local_ident(s, local_ident);
+        t30_get_transfer_statistics(s, &t);
+        t30_get_far_ident(s, far_ident);
+        t30_get_local_ident(s, local_ident);
         ast_log(LOG_DEBUG, "==============================================================================\n");
         ast_log(LOG_DEBUG, "Fax successfully received.\n");
         ast_log(LOG_DEBUG, "Remote station id: %s\n", far_ident);
@@ -89,11 +109,7 @@ static void phase_e_handler(t30_state_t *s, void *user_data, int result)
                       "FaxReceived", "Channel: %s\nExten: %s\nCallerID: %s\nRemoteStationID: %s\nLocalStationID: %s\nPagesTransferred: %i\nResolution: %i\nTransferRate: %i\nFileName: %s\n",
                       chan->name,
                       chan->exten,
-#if (ASTERISK_VERSION_NUM <= 010300)
-                      chan->callerid,
-#else
                       (chan->cid.cid_num)  ?  chan->cid.cid_num  :  "",
-#endif
                       far_ident,
                       local_ident,
                       t.pages_transferred,
@@ -111,7 +127,7 @@ static void phase_e_handler(t30_state_t *s, void *user_data, int result)
     else
     {
         ast_log(LOG_DEBUG, "==============================================================================\n");
-        ast_log(LOG_DEBUG, "Fax receive not successful.\n");
+        ast_log(LOG_DEBUG, "Fax receive not successful - result (%d) %s.\n", result, t30_completion_code_to_str(result));
         ast_log(LOG_DEBUG, "==============================================================================\n");
     }
 }
@@ -125,7 +141,7 @@ static void phase_d_handler(t30_state_t *s, void *user_data, int result)
     chan = (struct ast_channel *) user_data;
     if (result)
     {
-        fax_get_transfer_statistics(s, &t);
+        t30_get_transfer_statistics(s, &t);
         ast_log(LOG_DEBUG, "==============================================================================\n");
         ast_log(LOG_DEBUG, "Pages transferred:  %i\n", t.pages_transferred);
         ast_log(LOG_DEBUG, "Image size:         %i x %i\n", t.columns, t.rows);
@@ -143,15 +159,18 @@ static void phase_d_handler(t30_state_t *s, void *user_data, int result)
 static int rxfax_exec(struct ast_channel *chan, void *data)
 {
     int res = 0;
-    int count = 0;
-    int percentflag = 0;
-    char fil[256];
-    char tmp[256];
-    char *vdata;
+    char template_file[256];
+    char target_file[256];
+    char *s;
+    char *t;
+    char *v;
     char *x;
+    int option;
+    int len;
     int i;
     t30_state_t fax;
     int calling_party;
+    int verbose;
     int samples;
 
     struct localuser *u;
@@ -163,62 +182,68 @@ static int rxfax_exec(struct ast_channel *chan, void *data)
     
     uint8_t __buf[sizeof(uint16_t)*MAX_BLOCK_SIZE + 2*AST_FRIENDLY_OFFSET];
     uint8_t *buf = __buf + AST_FRIENDLY_OFFSET;
-    int len;
 
     if (chan == NULL)
     {
         ast_log(LOG_WARNING, "Fax receive channel is NULL. Giving up.\n");
         return -1;
     }
-    vdata = data;
+
+    span_set_message_handler(span_message);
 
     /* The next few lines of code parse out the filename and header from the input string */
-    if (!vdata)
+    if (data == NULL)
     {
         /* No data implies no filename or anything is present */
         ast_log(LOG_WARNING, "Rxfax requires an argument (filename)\n");
         return -1;
     }
     
-    for (i = 0;  vdata[i]  &&  (vdata[i] != ':')  &&  (vdata[i] != '|');  i++)
-    {
-        if ((vdata[i] == '%')  &&  (vdata[i + 1] == 'd'))
-            percentflag = 1;                      /* the wildcard is used */
-        
-        if (i == strlen(vdata))
-        {
-            ast_log(LOG_WARNING, "No extension found\n");
-            return -1;
-        }
-        fil[i] = vdata[i];
-    }
-    fil[i] = '\0';
-
     calling_party = FALSE;
-    if (vdata[i] == '|')
-    {
-        i++;
-        if (strcmp(vdata + 1, "caller") == 0)
-            calling_party = TRUE;
-    }
-    /* Done parsing */
+    verbose = FALSE;
+    target_file[0] = '\0';
 
-    /* These are to allow the use of the %d in the config file for a wild card of sort to
-       create a new file with the inputed name scheme */
-    if (percentflag)
+    for (option = 0, v = s = data;  v;  option++, s++)
     {
-        do
+        t = s;
+        v = strchr(s, '|');
+        s = (v)  ?  v  :  s + strlen(s);
+        strncpy((char *) buf, t, s - t);
+        buf[s - t] = '\0';
+        if (option == 0)
         {
-            snprintf(tmp, 256, fil, count);
-            count++;
+            /* The first option is always the file name */
+            len = s - t;
+            if (len > 255)
+                len = 255;
+            strncpy(target_file, t, len);
+            target_file[len] = '\0';
+            /* Allow the use of %d in the file name for a wild card of sorts, to
+               create a new file with the specified name scheme */
+            if ((x = strchr(target_file, '%'))  &&  x[1] == 'd')
+            {
+                strcpy(template_file, target_file);
+                i = 0;
+                do
+                {
+                    snprintf(target_file, 256, template_file, 1);
+                    i++;
+                }
+                while (ast_fileexists(target_file, "", chan->language) != -1);
+            }
+        }
+        else if (strncmp("caller", t, s - t) == 0)
+        {
+            calling_party = TRUE;
+        }
+        else if (strncmp("debug", t, s - t) == 0)
+        {
+            verbose = TRUE;
         }
-        while (ast_fileexists(tmp, "", chan->language) != -1);
-    }
-    else
-    {
-        strncpy(tmp, fil, 256 - 1);
     }
 
+    /* Done parsing */
+
     LOCAL_USER_ADD(u);
 
     if (chan->_state != AST_STATE_UP)
@@ -254,17 +279,18 @@ static int rxfax_exec(struct ast_channel *chan, void *data)
             }
         }
         fax_init(&fax, calling_party, NULL);
-
+        if (verbose)
+            fax.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
         x = pbx_builtin_getvar_helper(chan, "LOCALSTATIONID");
         if (x  &&  x[0])
-            fax_set_local_ident(&fax, x);
+            t30_set_local_ident(&fax, x);
         x = pbx_builtin_getvar_helper(chan, "LOCALHEADERINFO");
         if (x  &&  x[0])
-            fax_set_header_info(&fax, x);
-        fax_set_rx_file(&fax, tmp);
-        //fax_set_phase_b_handler(&fax, phase_b_handler, chan);
-        fax_set_phase_d_handler(&fax, phase_d_handler, chan);
-        fax_set_phase_e_handler(&fax, phase_e_handler, chan);
+            t30_set_header_info(&fax, x);
+        t30_set_rx_file(&fax, target_file, -1);
+        //t30_set_phase_b_handler(&fax, phase_b_handler, chan);
+        t30_set_phase_d_handler(&fax, phase_d_handler, chan);
+        t30_set_phase_e_handler(&fax, phase_e_handler, chan);
         while (ast_waitfor(chan, -1) > -1)
         {
             inf = ast_read(chan);
@@ -275,10 +301,10 @@ static int rxfax_exec(struct ast_channel *chan, void *data)
             }
             if (inf->frametype == AST_FRAME_VOICE)
             {
-                if (fax_rx_process(&fax, inf->data, inf->samples))
+                if (fax_rx(&fax, inf->data, inf->samples))
                     break;
                 samples = (inf->samples <= MAX_BLOCK_SIZE)  ?  inf->samples  :  MAX_BLOCK_SIZE;
-                len = fax_tx_process(&fax, (int16_t *) &buf[AST_FRIENDLY_OFFSET], samples);
+                len = fax_tx(&fax, (int16_t *) &buf[AST_FRIENDLY_OFFSET], samples);
                 if (len)
                 {
                     memset(&outf, 0, sizeof(outf));
@@ -288,6 +314,7 @@ static int rxfax_exec(struct ast_channel *chan, void *data)
                     outf.samples = len;
                     outf.data = &buf[AST_FRIENDLY_OFFSET];
                     outf.offset = AST_FRIENDLY_OFFSET;
+                    outf.src = "RxFAX";
                     if (ast_write(chan, &outf) < 0)
                     {
                         ast_log(LOG_WARNING, "Unable to write frame to channel; %s\n", strerror(errno));
@@ -314,6 +341,7 @@ static int rxfax_exec(struct ast_channel *chan, void *data)
             if (res)
                 ast_log(LOG_WARNING, "Unable to restore write format on '%s'\n", chan->name);
         }
+        fax_release(&fax);
     }
     else
     {
@@ -322,12 +350,14 @@ static int rxfax_exec(struct ast_channel *chan, void *data)
     LOCAL_USER_REMOVE(u);
     return res;
 }
+/*- End of function --------------------------------------------------------*/
 
 int unload_module(void)
 {
     STANDARD_HANGUP_LOCALUSERS;
     return ast_unregister_application(app);
 }
+/*- End of function --------------------------------------------------------*/
 
 int load_module(void)
 {
@@ -338,6 +368,7 @@ char *description(void)
 {
     return tdesc;
 }
+/*- End of function --------------------------------------------------------*/
 
 int usecount(void)
 {
@@ -345,8 +376,11 @@ int usecount(void)
     STANDARD_USECOUNT(res);
     return res;
 }
+/*- End of function --------------------------------------------------------*/
 
-char *key()
+char *key(void)
 {
     return ASTERISK_GPL_KEY;
 }
+/*- End of function --------------------------------------------------------*/
+/*- End of file ------------------------------------------------------------*/
