@@ -11,22 +11,28 @@
  * the GNU General Public License
  */
  
-#include <asterisk/lock.h>
-#include <asterisk/file.h>
-#include <asterisk/logger.h>
-#include <asterisk/channel.h>
-#include <asterisk/pbx.h>
-#include <asterisk/module.h>
-#include <asterisk/translate.h>
 #include <string.h>
 #include <stdlib.h>
-#include <stdint.h>
+#include <stdio.h>
+#include <inttypes.h>
 #include <pthread.h>
 #include <errno.h>
 #include <tiffio.h>
 
 #include <spandsp.h>
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision:$")
+
+#include "asterisk/lock.h"
+#include "asterisk/file.h"
+#include "asterisk/logger.h"
+#include "asterisk/channel.h"
+#include "asterisk/pbx.h"
+#include "asterisk/module.h"
+#include "asterisk/translate.h"
+
 static char *tdesc = "Trivial FAX Transmit Application";
 
 static char *app = "TxFAX";
@@ -34,11 +40,12 @@ static char *app = "TxFAX";
 static char *synopsis = "Send a FAX file";
 
 static char *descrip = 
-"  TxFAX(filename[|caller]):  Send a given TIFF file to the channel as a FAX.\n"
+"  TxFAX(filename[|caller][|debug]):  Send a given TIFF file to the channel as a FAX.\n"
 "The \"caller\" option makes the application behave as a calling machine,\n"
 "rather than the answering machine. The default behaviour is to behave as\n"
 "an answering machine.\n"
 "Uses LOCALSTATIONID to identify itself to the remote end.\n"
+"     LOCALHEADERINFO to generate a header line on each page.\n"
 "Sets REMOTESTATIONID to the receiver CSID.\n"
 "Returns -1 when the user hangs up, or if the file does not exist.\n"
 "Returns 0 otherwise.\n";
@@ -49,6 +56,20 @@ LOCAL_USER_DECL;
 
 #define MAX_BLOCK_SIZE 240
 
+static void span_message(int level, const char *msg)
+{
+    int ast_level;
+    
+    if (level == SPAN_LOG_WARNING)
+        ast_level = __LOG_WARNING;
+    else if (level == SPAN_LOG_WARNING)
+        ast_level = __LOG_WARNING;
+    else
+        ast_level = __LOG_DEBUG;
+    ast_log(ast_level, __FILE__, __LINE__, __PRETTY_FUNCTION__, msg);
+}
+/*- End of function --------------------------------------------------------*/
+
 static void t30_flush(t30_state_t *s, int which)
 {
     //TODO:
@@ -61,15 +82,15 @@ static void phase_e_handler(t30_state_t *s, void *user_data, int result)
     char far_ident[21];
     
     chan = (struct ast_channel *) user_data;
-    if (result)
+    if (result == T30_ERR_OK)
     {
-        fax_get_far_ident(s, far_ident);
+        t30_get_far_ident(s, far_ident);
         pbx_builtin_setvar_helper(chan, "REMOTESTATIONID", far_ident);
     }
     else
     {
         ast_log(LOG_DEBUG, "==============================================================================\n");
-        ast_log(LOG_DEBUG, "Fax send not successful.\n");
+        ast_log(LOG_DEBUG, "Fax send not successful - result (%d) %s.\n", result, t30_completion_code_to_str(result));
         ast_log(LOG_DEBUG, "==============================================================================\n");
     }
 }
@@ -78,15 +99,16 @@ static void phase_e_handler(t30_state_t *s, void *user_data, int result)
 static int txfax_exec(struct ast_channel *chan, void *data)
 {
     int res = 0;
-    int count = 0;
-    int percentflag = 0;
-    char fil[256];
-    char tmp[256];
-    char *vdata;
-    char *localident;
-    int i;
+    char source_file[256];
+    char *x;
+    char *s;
+    char *t;
+    char *v;
+    int option;
+    int len;
     t30_state_t fax;
     int calling_party;
+    int verbose;
     int samples;
     
     struct localuser *u;
@@ -98,61 +120,55 @@ static int txfax_exec(struct ast_channel *chan, void *data)
     
     uint8_t __buf[sizeof(uint16_t)*MAX_BLOCK_SIZE + 2*AST_FRIENDLY_OFFSET];
     uint8_t *buf = __buf + AST_FRIENDLY_OFFSET;
-    int len;
 
     if (chan == NULL)
     {
         ast_log(LOG_WARNING, "Fax transmit channel is NULL. Giving up.\n");
         return -1;
     }
-    vdata = data;
+
+    span_set_message_handler(span_message);
 
     /* The next few lines of code parse out the filename and header from the input string */
-    if (!vdata)
+    if (data == NULL)
     {
         /* No data implies no filename or anything is present */
         ast_log(LOG_WARNING, "Txfax requires an argument (filename)\n");
         return -1;
     }
     
-    for (i = 0;  vdata[i]  &&  (vdata[i] != ':')  &&  (vdata[i] != '|');  i++)
-    {
-        if ((vdata[i] == '%')  &&  (vdata[i + 1] == 'd'))
-            percentflag = 1;                      /* the wildcard is used */
-        
-        if (i == strlen(vdata))
-        {
-            ast_log(LOG_WARNING, "No extension found\n");
-            return -1;
-        }
-        fil[i] = vdata[i];
-    }
-    fil[i] = '\0';
     calling_party = FALSE;
-    if (vdata[i] == '|')
-    {
-        i++;
-        if (strcmp(vdata + i, "caller") == 0)
-            calling_party = TRUE;
-    }
-    /* Done parsing */
+    verbose = FALSE;
+    source_file[0] = '\0'; 
 
-    /* These are to allow the use of the %d in the config file for a wild card of sort to
-       create a new file with the inputed name scheme */
-    if (percentflag)
+    for (option = 0, v = s = data;  v;  option++, s++)
     {
-        do
+        t = s;
+        v = strchr(s, '|');
+        s = (v)  ?  v  :  s + strlen(s);
+        strncpy((char *) buf, t, s - t);
+        buf[s - t] = '\0';
+        if (option == 0)
         {
-            snprintf(tmp, 256, fil, count);
-            count++;
+            /* The first option is always the file name */
+            len = s - t;
+            if (len > 255)
+                len = 255;
+            strncpy(source_file, t, len);
+            source_file[len] = '\0';
+        }
+        else if (strncmp("caller", t, s - t) == 0)
+        {
+            calling_party = TRUE;
+        }
+        else if (strncmp("debug", t, s - t) == 0)
+        {
+            verbose = TRUE;
         }
-        while (ast_fileexists(tmp, "", chan->language) != -1);
-    }
-    else
-    {
-        strncpy(tmp, fil, 256 - 1);
     }
 
+    /* Done parsing */
+
     LOCAL_USER_ADD(u);
 
     if (chan->_state != AST_STATE_UP)
@@ -188,13 +204,19 @@ static int txfax_exec(struct ast_channel *chan, void *data)
             }
         }
         fax_init(&fax, calling_party, NULL);
+        if (verbose)
+	    fax.logging.level = SPAN_LOG_SHOW_SEVERITY | SPAN_LOG_SHOW_PROTOCOL | SPAN_LOG_FLOW;
 
-        localident = pbx_builtin_getvar_helper(chan, "LOCALSTATIONID");
-        fax_set_local_ident(&fax, localident);
-        fax_set_tx_file(&fax, tmp);
-        //fax_set_phase_b_handler(&fax, phase_b_handler, chan);
-        //fax_set_phase_d_handler(&fax, phase_d_handler, chan);
-        fax_set_phase_e_handler(&fax, phase_e_handler, chan);
+        x = pbx_builtin_getvar_helper(chan, "LOCALSTATIONID");
+        if (x  &&  x[0])
+            t30_set_local_ident(&fax, x);
+        x = pbx_builtin_getvar_helper(chan, "LOCALHEADERINFO");
+        if (x  &&  x[0])
+            t30_set_header_info(&fax, x);
+        t30_set_tx_file(&fax, source_file, -1, -1);
+        //t30_set_phase_b_handler(&fax, phase_b_handler, chan);
+        //t30_set_phase_d_handler(&fax, phase_d_handler, chan);
+        t30_set_phase_e_handler(&fax, phase_e_handler, chan);
         while (ast_waitfor(chan, -1) > -1)
         {
             inf = ast_read(chan);
@@ -205,10 +227,10 @@ static int txfax_exec(struct ast_channel *chan, void *data)
             }
             if (inf->frametype == AST_FRAME_VOICE)
             {
-                if (fax_rx_process(&fax, inf->data, inf->samples))
+                if (fax_rx(&fax, inf->data, inf->samples))
                     break;
                 samples = (inf->samples <= MAX_BLOCK_SIZE)  ?  inf->samples  :  MAX_BLOCK_SIZE;
-                len = fax_tx_process(&fax, (int16_t *) &buf[AST_FRIENDLY_OFFSET], samples);
+                len = fax_tx(&fax, (int16_t *) &buf[AST_FRIENDLY_OFFSET], samples);
                 if (len)
                 {
                     memset(&outf, 0, sizeof(outf));
@@ -244,6 +266,7 @@ static int txfax_exec(struct ast_channel *chan, void *data)
             if (res)
                 ast_log(LOG_WARNING, "Unable to restore write format on '%s'\n", chan->name);
         }
+        fax_release(&fax);
     }
     else
     {
@@ -252,22 +275,26 @@ static int txfax_exec(struct ast_channel *chan, void *data)
     LOCAL_USER_REMOVE(u);
     return res;
 }
+/*- End of function --------------------------------------------------------*/
 
 int unload_module(void)
 {
     STANDARD_HANGUP_LOCALUSERS;
     return ast_unregister_application(app);
 }
+/*- End of function --------------------------------------------------------*/
 
 int load_module(void)
 {
     return ast_register_application(app, txfax_exec, synopsis, descrip);
 }
+/*- End of function --------------------------------------------------------*/
 
 char *description(void)
 {
     return tdesc;
 }
+/*- End of function --------------------------------------------------------*/
 
 int usecount(void)
 {
@@ -276,8 +303,11 @@ int usecount(void)
     STANDARD_USECOUNT(res);
     return res;
 }
+/*- End of function --------------------------------------------------------*/
 
-char *key()
+char *key(void)
 {
     return ASTERISK_GPL_KEY;
 }
+/*- End of function --------------------------------------------------------*/
+/*- End of file ------------------------------------------------------------*/
